@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+"A console-based gmail e-mail reader."
 # -*- coding: utf-8 -*-
 
 # Copyright (c) 2008
@@ -39,12 +40,19 @@ import HTMLParser
 
 from getpass import getpass
 from htmlentitydefs import entitydefs
+from mimify import mime_decode_header
 
 import libgmail
 
+from MIMEParser import MIMEParser
+import configvars as conf
+from Config import Config
+from tabler import tabler
+
 # Time (in seconds) to wait between e-mail checks
 TIMEOUT = 10
 
+# These are the constants identifying the commands
 LIST_FOLDERS = 'lf'
 LIST_EMAILS = 'lm'
 ENTER_FOLDER = 'cd'
@@ -59,10 +67,12 @@ QUIT = 'q'
 
 
 class NoCommandError(Exception):
+    """Command doesn't exist."""
     pass
 
 
 class ExecutionError(Exception):
+    """something went wrong while executing a command."""
     def __init__(self, message):
         self.message = message
 
@@ -70,7 +80,12 @@ class ExecutionError(Exception):
         return repr(self.message)
 
 
-class AuthorFieldParser(HTMLParser.HTMLParser):
+class FieldParser(HTMLParser.HTMLParser):
+    """This is an HTML Parser which ignores any tag except span. Only the data
+    between <span> is stored in the text attribute.
+    
+    This is handy for filtering out the HTML on subject and author fields."""
+
     def __init__(self):
         HTMLParser.HTMLParser.__init__(self)
         self.inside_span = False
@@ -97,6 +112,7 @@ class AccountState:
     """This class have variables that must be shared along the commands. The
     ReadEmail command must know what were the last messages displayed to the
     user, for instance."""
+
     def __init__(self):
         # starting directory is not a label
         self.current_dir = 'inbox'
@@ -109,16 +125,25 @@ class AccountState:
 class Command:
     """This is the basic unit of the program. All the user does is type down
     commands which will give him messages on the screen."""
+
     def __init__(self, s, state, acc):
+        """__init__(self, string, AccountState, GmailAccount)
+
+        This will just save the current AccountState and GmailAccount to the
+        object without doing anything."""
         self.state = state
         self.acc = acc
 
     def execute(self):
+        """This method will execute the command, potentially changing the
+        AccountState that was passed to the object's __init__."""
         pass
 
 
 class ListFolders(Command):
     def execute(self):
+        # folders like inbox, all, etc are threated differently than labels by
+        # libgmail.
         if self.acc.getLabelNames():
             all_labels = libgmail.STANDARD_FOLDERS + self.acc.getLabelNames()
         else:
@@ -131,6 +156,10 @@ class ListFolders(Command):
 
 class Archive(Command):
     def __init__(self, s, state, acc):
+        """__init__(self, string, AccountState, GmailAccount)
+
+        Differently than the standard __init__, it will take the string s into
+        account as the command's arguments."""
         Command.__init__(self, s, state, acc)
         self.arg = s.strip()
 
@@ -146,6 +175,10 @@ class Archive(Command):
 
 class ReportSpam(Command):
     def __init__(self, s, state, acc):
+        """__init__(self, string, AccountState, GmailAccount)
+
+        Differently than the standard __init__, it will take the string s into
+        account as the command's arguments."""
         Command.__init__(self, s, state, acc)
         self.arg = s.strip()
 
@@ -161,6 +194,10 @@ class ReportSpam(Command):
 
 class EnterFolder(Command):
     def __init__(self, s, state, acc):
+        """__init__(self, string, AccountState, GmailAccount)
+
+        Differently than the standard __init__, it will take the string s into
+        account as the command's arguments."""
         Command.__init__(self, s, state, acc)
         self.arg = s.strip()
 
@@ -207,16 +244,23 @@ class ListEmails(Command):
         for i, c in enumerate(conversations):
             t.append((str(i),
                       ['', 'N'][bool(c.unread)],
-                      self.__fix_html(self.__fix_encoding(c.authors)),
-                      self.__fix_html(self.__fix_encoding(c.subject)),
+                      self.__fix_all(c.authors),
+                      self.__fix_all(c.subject),
                      )
                     )
 
-        for line in self.__tabler(t):
+        for line in tabler(t):
             print line
 
     #XXX: helper functions. They're ok for now, but I might want to revist them,
     #     maybe write something better as -- if -- the project moves
+    def __fix_all(self, s):
+        s = self.__fix_encoding(s)
+        s = self.__fix_html(s)
+        s = self.__entitytoletter(s)
+
+        return s
+
     def __entitytoletter(self, s):
         newdefs = []
         for k, v in entitydefs.items():
@@ -228,14 +272,11 @@ class ListEmails(Command):
         return s
 
     def __fix_html(self, s):
-        parser = AuthorFieldParser()
+        parser = FieldParser()
         try:
             parser.feed(s)
             parser.close()
         except HTMLParser.HTMLParseError:
-            #XXX: I gotta make some better html handling some day. The shouldn't
-            #be any HTML in the title, really, maybe if I don't change the
-            #HTML entities to actual characters before hand, who knows...
             parser.text = s
 
         return parser.text
@@ -252,123 +293,52 @@ class ListEmails(Command):
                 final.append(s[i])
                 i += 1
 
-        return self.__entitytoletter(''.join(final).strip())
+        return ''.join(final).strip()
 
-    def __greatest_len(self, l):
-        return max(map(len, l))
 
-    def __compose_field(self, lines):
-        x = self.__greatest_len(lines) + 1
-        fields = []
-        for line in lines:
-            fields.append(line+((x-len(line))*' '))
+import re
+def _get_email(mail):
+    """_get_email(string) -> string
 
-        return fields
+    Returns the stripped out and all lower case e-mail address (without any
+    names in the form "My Name")"""
 
-    def __concat(self, t1, t2):
-        t = []
-        for a, b in zip(t1,t2):
-            t.append(a+b)
-
-        return t
-
-    def __tabler(self, t):
-        if not t:
-            return []
-
-        table = []
-        table = self.__compose_field([x[0] for x in t])
-        table = self.__concat(table, self.__compose_field([x[1] for x in t]))
-        table = self.__concat(table, self.__compose_field([x[2] for x in t]))
-
-        indentsize = self.__greatest_len([x[0] for x in t]) +\
-                     self.__greatest_len([x[1] for x in t])
-        table = self.__concat(table,
-                              ['\n'+((2+indentsize)*' ')+x[3] for x in t])
-
-        return table
+    res = re.findall(r'"?.+"? +<(.+@.+)>', mail)
+    if res:
+        return res[0].strip().lower()
+    else:
+        return mail.strip().lower()
 
 
 class ReadEmail(Command):
+    EMAIL_DIVISOR = '\n\n'+(80*'-')+'\n\n'
+
     def __init__(self, s, state, acc):
+        """__init__(self, string, AccountState, GmailAccount)
+
+        Differently than the standard __init__, it will take the string s into
+        account as the command's arguments."""
         Command.__init__(self, s, state, acc)
         self.arg = s.strip()
 
-    def __select_payload(self, payload):
-        # looking for the real message
-        for msg in payload:
-            if msg.get_content_type() == 'multipart/alternative':
-                payload = msg.get_payload()
-                break
-
-        # let's search for html and plain stuff
-        html = None
-        plain = None
-        for msg in payload:
-            if msg.get_content_type() == 'text/html':
-                html = msg.get_payload(decode=True)
-                charset = msg.get_content_charset()
-            elif msg.get_content_type() == 'text/plain':
-                plain = msg.get_payload(decode=True)
-                charset = msg.get_content_charset()
-
-        p = subprocess.Popen(['html2text'],
-                             stdin=subprocess.PIPE,
-                             stdout=subprocess.PIPE)
-        if html:
-            (output, err) = p.communicate(html)
-            if charset:
-                try:
-                    return output.decode(charset).encode('utf-8')
-                except UnicodeDecodeError:
-                    # We can't always trust the information we're given :-/
-                    return output
-            else:
-                return output
-        elif plain:
-            if charset:
-                try:
-                    return plain.decode(charset).encode('utf-8')
-                except UnicodeDecodeError:
-                    # We can't always trust the information we're given :-/
-                    return plain
-            else:
-                return plain
-        else:
-            tmp = payload[0].get_payload(decode=True)
-            charset = payload[0].get_content_charset()
-            if charset:
-                try:
-                    return tmp.decode(charset).encode('utf-8')
-                except UnicodeDecodeError:
-                    # We can't always trust the information we're given :-/
-                    return tmp
-            else:
-                return tmp
-
     def __print_field(self, msg, field):
         value = msg.get(field)
         if value:
+            value = mime_decode_header(value)
+            value = value.decode('iso8859-1').encode('utf-8')
             return field.capitalize()+': '+value
         else:
             return ''
 
-    EMAIL_DIVISOR = '\n\n'+(80*'-')+'\n\n'
-
-    def __formating(self, text, msgid):
+    def __format(self, text, msgid):
         msg = email.message_from_string(text)
         mget = lambda field: self.__print_field(msg, field)
-        if msg.is_multipart():
-            body = self.__select_payload(msg.get_payload())
-        else:
-            body = msg.get_payload(decode=True)
-            charset = msg.get_content_charset()
-            if charset:
-                try:
-                    body = body.decode(charset).encode('utf-8')
-                except UnicodeDecodeError:
-                    # We can't always trust the information we're given :-/
-                    return body
+
+        mp = MIMEParser(text)
+        body = mp.body
+        if mp.forward:
+            body += '\n\n' + mp.forward
+
         fields = [mget('to'),
                   mget('cc'),
                   mget('from'),
@@ -393,6 +363,7 @@ class ReadEmail(Command):
 
         #extracting information
         frm = msg.get('From', '')
+        reply_to = msg.get('Reply-To', '')
         cc = msg.get('CC', '').split(',')
         to = msg.get('To', '').split(',')
         id = msg.get('Message-id', '')
@@ -404,11 +375,25 @@ class ReadEmail(Command):
         #setting up the reply
         body = self.__add_quote(body)
         body = ("On %s, %s wrote:\n" % (msg.get('Date'), frm)) + body
+
         newmsg = email.message_from_string(body)
-        newmsg['To'] = frm
-        receivers = [x for x in to if self.acc.name not in x]
-        receivers.extend(cc)
+
+        if reply_to:
+            newmsg['To'] = reply_to.strip()
+        else:
+            newmsg['To'] = frm.strip()
+
+        receivers = set(to).union(set(cc))
+        # remove any string without an email
+        receivers = (x.strip() for x in receivers if x.strip() != '')
+        # remove my own email from the list
+        myemail = _get_email(self.acc.name)
+        receivers = (x for x in receivers if myemail != _get_email(x))
+        # remove the email we're using in the To field from the CC field
+        hisemail = _get_email(newmsg['To'])
+        receivers = (x for x in receivers if hisemail != _get_email(x))
         newmsg['CC'] =  ', '.join(receivers)
+
         newmsg['In-reply-to'] = id
         newmsg['Subject'] = subject
 
@@ -422,24 +407,29 @@ class ReadEmail(Command):
         except IndexError:
             raise ExecutionError("Invalid thread number")
 
-        f = open(TMP, 'w')
+        f = open(conf.TMP, 'w')
         for msg in conversation:
-            print>>f, self.__formating(msg.source, msg.id)
+            print>>f, self.__format(msg.source, msg.id)
         f.close()
-        mtime = os.path.getmtime(TMP)
+        mtime = os.path.getmtime(conf.TMP)
 
-        subprocess.call([EDITOR, TMP])
+        os.system('%s %s' % (conf.EDITOR, conf.TMP))
 
-        if mtime != os.path.getmtime(TMP):
-            shutil.copy(TMP, DRAFT)
-            text = self.__reply_maker(open(DRAFT).read())
-            f = open(DRAFT, 'w')
+        if mtime != os.path.getmtime(conf.TMP):
+            shutil.copy(conf.TMP, conf.DRAFT)
+            text = self.__reply_maker(open(conf.DRAFT).read())
+            f = open(conf.DRAFT, 'w')
             f.write(text)
             f.close()
 
 
 class WaitEmail(Command):
     def __init__(self, s, state, acc):
+        """__init__(self, string, AccountState, GmailAccount)
+
+        Differently than the standard __init__, it will take the string s into
+        account as the command's arguments. But instead of using the whole
+        string as one argument, it will split it into several arguments."""
         Command.__init__(self, s, state, acc)
         self.arg = [x.strip() for x in s.split()]
 
@@ -478,7 +468,7 @@ class WaitEmail(Command):
 
 class SendEmail(Command):
     def execute(self):
-        text = open(DRAFT).read()
+        text = open(conf.DRAFT).read()
         attrs = email.message_from_string(text)
         msg = libgmail.GmailComposedMessage(attrs.get('to'),
                                             attrs.get('subject'),
@@ -490,7 +480,7 @@ class SendEmail(Command):
 
 class ComposeEmail(Command):
     def execute(self):
-        subprocess.call([EDITOR, DRAFT])
+        os.system('%s %s' % (conf.EDITOR, conf.DRAFT))
 
 
 class Help(Command):
@@ -555,34 +545,6 @@ class CommandFactory:
             raise NoCommandError()
 
 
-class Config:
-    """This is a readonly config file handler."""
-    def __init__(self, fname):
-        self.attrs = {}
-        try:
-            lines = open(fname).read().split('\n')
-        except:
-            open(fname, 'w').close()
-            os.chmod(fname, 0600)
-        else:
-            for line in lines:
-                arg = line.split('=')
-                key = arg[0].strip()
-                self.attrs[key] = reduce(str.__add__, arg[1:], '').strip()
-    
-    def get(self, key, default = lambda: None):
-        """get(key, defaultfun) -> string|None
-
-        This method was created allowing for lazy evaluation, you should pass a
-        function as a parameter which, when evaluated will return the default
-        value. This allow for very small code for when we want to read user
-        input as a default."""
-        if self.attrs.has_key(key):
-            return self.attrs[key]
-        else:
-            return default()
-
-
 def main():
     conf = Config(os.path.expanduser('~/.gmailreader/config'))
     email = conf.get('username', lambda: raw_input("Username: "))
@@ -624,20 +586,6 @@ def main():
 
 if __name__ == '__main__':
     try:
-        # global variables setup
-        if os.system('mkdir -p ~/.gmailreader'):
-            sys.stderr.write('Unable to create ~/.gmailreader\n')
-            raise SystemExit, 1
-
-        DRAFT = os.path.expanduser('~/.gmailreader/draft')
-        TMP = os.path.expanduser('~/.gmailreader/tmp')
-
-        EDITOR=Config(os.path.expanduser('~/.gmailreader/config')).get('editor')
-        if not EDITOR:
-            EDITOR = os.getenv('EDITOR')
-        if not EDITOR:
-            EDITOR = 'vi'
-
         main()
     except KeyboardInterrupt:
         print
